
local AchievementIcon = "Interface\\AddOns\\Overachiever\\AchShield";
local tooltip_complete = { r = 0.2, g = 0.5, b = 0.2 }
local tooltip_incomplete = { r = 1, g = 0.1, b = 0.1 }

local function sethook(frame, script, handler)
  local prev = frame:GetScript(script)
  if (prev) then
    frame:HookScript(script, handler)
  else
    frame:SetScript(script, handler)
  end
end

local function isCriteria(achID, name)
  local n, t, complete
  for i=1,GetAchievementNumCriteria(achID) do
    n, t, complete = GetAchievementCriteriaInfo(achID, i)
    if (n == name) then  return true, complete;  end
  end
end


-- UNIT TOOLTIP HOOK
----------------------

-- "To All The Squirrels I've Loved Before" achievement:
local ACH_LoveCritters_Complete
local function CritterCheck(name, unit)
  if (ACH_LoveCritters_Complete or not Overachiever_Settings.CritterTip_loved) then  return;  end
  if (name and unit and UnitCreatureType(unit) == OVERACHIEVER_STRINGS_CRITTER) then
    if (select(4, GetAchievementInfo(OVERACHIEVER_ACHID.LoveCritters))) then
      ACH_LoveCritters_Complete = true;
      return;
    end
    local isCrit, complete = isCriteria(OVERACHIEVER_ACHID.LoveCritters, name)
    if (isCrit) then
      return complete and OVERACHIEVER_STRINGS_ACH_LOVECRITTERS_COMPLETE or OVERACHIEVER_STRINGS_ACH_LOVECRITTERS_INCOMPLETE, complete
    end
  end
end

local RaceClassAch = {
  LetItSnow = { "LetItSnow_flaked", OVERACHIEVER_STRINGS_ACH_LETITSNOW_COMPLETE, OVERACHIEVER_STRINGS_ACH_LETITSNOW_INCOMPLETE },
  FistfulOfLove = { "FistfulOfLove_petals", OVERACHIEVER_STRINGS_ACH_FISTFULOFLOVE_COMPLETE, OVERACHIEVER_STRINGS_ACH_FISTFULOFLOVE_INCOMPLETE },
};

local function RaceClassCheck(ach, text)
  local id = OVERACHIEVER_ACHID[ach]
  if (select(4, GetAchievementInfo(id))) then
    RaceClassAch[ach] = nil;
    return;
  end
  local isCrit, complete = isCriteria(id, text)
  if (isCrit) then
    return complete and RaceClassAch[ach][2] or RaceClassAch[ach][3], complete
  end
end

local function ExamineSetUnit(tooltip)
  tooltip = tooltip or GameTooltip  -- Workaround since another addon is known to break this
  local name, unit = tooltip:GetUnit()
  local text, complete = CritterCheck(name, unit)

  if (not text and UnitIsPlayer(unit)) then
    local raceclass = UnitRace(unit).." "..UnitClass(unit)
    for key,tab in pairs(RaceClassAch) do
      if (Overachiever_Settings[ tab[1] ]) then
        text, complete = RaceClassCheck(key, raceclass)
        if (text) then  break;  end
      end
    end
  end

  if (text) then
    local r, g, b
    if (complete) then
      r, g, b = tooltip_complete.r, tooltip_complete.g, tooltip_complete.b
    else
      r, g, b = tooltip_incomplete.r, tooltip_incomplete.g, tooltip_incomplete.b
    end
    tooltip:AddLine(text, r, g, b)
    tooltip:AddTexture(AchievementIcon)
    tooltip:Show()
  end
end

sethook(GameTooltip, "OnTooltipSetUnit", ExamineSetUnit)


-- WORLD OBJECT TOOLTIP HOOK
------------------------------

local WorldObjAch = {
  WellRead = { "WellReadTip_read", OVERACHIEVER_STRINGS_ACH_WELLREAD_COMPLETE, OVERACHIEVER_STRINGS_ACH_WELLREAD_INCOMPLETE },
  HigherLearning = { "WellReadTip_read", OVERACHIEVER_STRINGS_ACH_WELLREAD_COMPLETE, OVERACHIEVER_STRINGS_ACH_WELLREAD_INCOMPLETE },
  Scavenger = { "AnglerTip_fished", OVERACHIEVER_STRINGS_ACH_ANGLER_COMPLETE, OVERACHIEVER_STRINGS_ACH_ANGLER_INCOMPLETE },
  OutlandAngler = { "AnglerTip_fished", OVERACHIEVER_STRINGS_ACH_ANGLER_COMPLETE, OVERACHIEVER_STRINGS_ACH_ANGLER_INCOMPLETE },
  NorthrendAngler = { "AnglerTip_fished", OVERACHIEVER_STRINGS_ACH_ANGLER_COMPLETE, OVERACHIEVER_STRINGS_ACH_ANGLER_INCOMPLETE },
};

local function WorldObjCheck(ach, text)
  local id = OVERACHIEVER_ACHID[ach]
  if (select(4, GetAchievementInfo(id))) then
    WorldObjAch[ach] = nil;
    return;
  end
  local isCrit, complete = isCriteria(id, text)
  if (isCrit) then
    return complete and WorldObjAch[ach][2] or WorldObjAch[ach][3], complete
  end
end

local function ExamineOneLiner(tooltip)
-- Unfortunately, we couldn't find a "GameTooltip:SetWorldObject" or similar type of thing, so we have to check for
-- these sorts of tooltips in a less direct way.
  tooltip = tooltip or GameTooltip  -- Workaround since another addon is known to break this
  if (tooltip:NumLines() == 1) then
    local n = tooltip:GetName()
    if (getglobal(n.."TextRight1"):GetText()) then  return;  end
    local tiptext = getglobal(n.."TextLeft1"):GetText()
    local text, complete
    for key,tab in pairs(WorldObjAch) do
      if (Overachiever_Settings[ tab[1] ]) then
        text, complete = WorldObjCheck(key, tiptext)
        if (text) then  break;  end
      end
    end
    if (text) then
      local r, g, b
      if (complete) then
        r, g, b = tooltip_complete.r, tooltip_complete.g, tooltip_complete.b
      else
        r, g, b = tooltip_incomplete.r, tooltip_incomplete.g, tooltip_incomplete.b
      end
      tooltip:AddLine(text, r, g, b)
      tooltip:AddTexture(AchievementIcon)
      tooltip:Show()
    end
  end
end

sethook(GameTooltip, "OnShow", ExamineOneLiner)
